package $package;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.services.AccessDeniedException;
import org.ow2.weblab.core.services.Analyser;
import org.ow2.weblab.core.services.ContentNotAvailableException;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.ServiceNotConfiguredException;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.UnsupportedRequestException;
import org.ow2.weblab.core.services.analyser.ProcessArgs;

/**
 * Template for WebLab service test which uses Analyser interface
 * 
 * @author WebLab Team
 */
public class $classNameTest {


	/**
	 * The instance of logger used inside $fileName
	 */
	private final Log logger = LogFactory.getLog(this.getClass());


	/**
	 * The instance of integer used to define number of thread called
	 */
	private final static int NB_THREAD = 3;


	/**
	 * The instance of longer is used such as timeout of thread
	 */
	private final static long TIMEOUT = 5L;


	/**
	 * The instance of string is path where are the resources of corpus
	 */
	private final static String PATH_CORPUS = "src/test/resources/corpus";


	/**
	 * Instance of service which implement a Analyser
	 */
	private $className analyser;


	/**
	 * Initialisation before tests
	 */
	@Before
	public void init() {
		this.analyser = new $className();
	}


	/**
	 * Exception tests
	 */
	@Test(expected = InvalidParameterException.class)
	public void testNullProcessArgs() throws Exception {
		this.analyser.process(null);
	}


	/**
	 * Exception tests
	 */
	@Test(expected = InvalidParameterException.class)
	public void testNullResourceProcessArgs() throws Exception {
		this.analyser.process(new ProcessArgs());
	}


	/**
	 * Multi-thread tests
	 * 
	 * TODO Delete expected exception when method process will be implemented.
	 * For the moment, method throws an exception instead of return a ProcessReturn.
	 */
	@Test(expected = ExecutionException.class)
	public void testAnalyserMultiThread() throws Exception {
		final List<Document> docs = this.getDocuments();
		final List<Callable<Resource>> analysers = new ArrayList<Callable<Resource>>();

		this.logger.info("Begin thread creation.");

		for (int i = 1; i <= NB_THREAD; i++) {
			final Document document = docs.get((int) (Math.random() * (docs.size())));
			final ProcessArgs analyserArgs = new ProcessArgs();
			analyserArgs.setResource(document);
			analyserArgs.setUsageContext(null);
			final AnalyserMultiThread thread = new AnalyserMultiThread(this.analyser, analyserArgs);
			analysers.add(thread);
		}

		this.logger.info("Thread created.");

		final ExecutorService executor = Executors.newFixedThreadPool(NB_THREAD);
		final List<Future<Resource>> results = executor.invokeAll(analysers, TIMEOUT, TimeUnit.MINUTES);

		for (final Future<Resource> future : results) {
			Assert.assertTrue(future.isDone());
			Assert.assertFalse(future.isCancelled());
			Assert.assertNotNull(future.get());
		}

		this.logger.info("Thread executed.");
		executor.shutdownNow();		
	}


	/**
	 * Method which allows to obtain resources
	 */
	private List<Document> getDocuments() throws WebLabCheckedException {
		final List<Document> docs = new ArrayList<Document>();
		for (final File file : new File(PATH_CORPUS).listFiles()) {
			if (file.isFile() && file.getName().endsWith(".xml")) {
				docs.add(new WebLabMarshaller().unmarshal(file, Document.class));
			}
		}
		return docs;
	}


	/**
	 * Internal class used to realize multi-thread tests with $className
	 */
	class AnalyserMultiThread implements Callable<Resource> {


		final private Analyser analyserMT;


		final private ProcessArgs processArgs;


		public AnalyserMultiThread(final Analyser analyserMT, final ProcessArgs processArgs) {
			this.analyserMT = analyserMT;
			this.processArgs = processArgs;
		}


		@Override
		public Resource call() throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException, InvalidParameterException,
				ServiceNotConfiguredException, UnexpectedException, UnsupportedRequestException {
			return this.getAnalyser().process(this.processArgs).getResource();
		}


		public Analyser getAnalyser() {
			return this.analyserMT;
		}
	}

}